# -*-coding:Utf-8 -*

# ---------------------------------------------------------------------
#
# Copyright (c) CREATIS : www.creatis.insa-lyon.fr
# Centre de Recherche en Acquisition et Traitement de l'Image pour la Santé
# 
#  This software is governed by the CeCILL-B license under French law and
#  abiding by the rules of distribution of free software. You can  use,
#  modify and/ or redistribute the software under the terms of the CeCILL-B
#  license as circulated by CEA, CNRS and INRIA at the following URL
#  www.cecill.info/licences/Licence_CeCILL-B_V1-en.html
#  or in the file LICENCE.txt.
#
#  As a counterpart to the access to the source code and  rights to copy,
#  modify and redistribute granted by the license, users are provided only
#  with a limited warranty  and the software's author,  the holder of the
#  economic rights,  and the successive licensors  have only  limited
#  liability.
#
#  The fact that you are presently reading this means that you have had
#  knowledge of the CeCILL-B license and that you accept its terms.
# ------------------------------------------------------------------------

from nmrglue import bruker
from os.path import dirname

# =======================================================================

def load(file_name):
    meta, data = bruker.read(dirname(file_name))
    udic = bruker.guess_udic(meta, data)
    if data.ndim == 1:
        data.shape = (1, data.shape[0])
    elif data.ndim == 2 and udic[0] != udic[1]:
        print("We're not supposed to handle hetero nuc signals")
    elif data.ndim > 2:
        raise NotImplementedError("Dimension higher than 2 is not supported.")
    meta = fromNgToMeta(data, udic)

    return data, meta

def save(file_name):
    raise NotImplementedError("save for bruker not implemented.")

# =======================================================================

def fromNgToMeta(ng_data, ng_dic)->dict:
    """
    Transform a nmrGlue output into a supported dic format.

    Since we're supposed to only support mono nuclear signals, we can only pay
    attention to ng_dic[0]
    """

    ng_dic['size'] = ng_data.shape[1]
    ng_dic['step'] = 1 / ng_dic[0]['sw']
    ng_dic['transFreq'] = ng_dic[0]['obs'] * 1e6
    ng_dic['B0'] = 0 # is not needed
    ng_dic['ppmref'] = 0 # same
    ng_dic['nbSignal'] = ng_data.shape[0]
    return ng_dic

# =======================================================================

if __name__ == '__main__':
    pass
