# -*-coding:Utf-8 -*

# ---------------------------------------------------------------------
#
# Copyright (c) CREATIS : www.creatis.insa-lyon.fr
# Centre de Recherche en Acquisition et Traitement de l'Image pour la Santé
# 
#  This software is governed by the CeCILL-B license under French law and
#  abiding by the rules of distribution of free software. You can  use,
#  modify and/ or redistribute the software under the terms of the CeCILL-B
#  license as circulated by CEA, CNRS and INRIA at the following URL
#  www.cecill.info/licences/Licence_CeCILL-B_V1-en.html
#  or in the file LICENCE.txt.
#
#  As a counterpart to the access to the source code and  rights to copy,
#  modify and redistribute granted by the license, users are provided only
#  with a limited warranty  and the software's author,  the holder of the
#  economic rights,  and the successive licensors  have only  limited
#  liability.
#
#  The fact that you are presently reading this means that you have had
#  knowledge of the CeCILL-B license and that you accept its terms.
# ------------------------------------------------------------------------

"""
Load and save function for HDF5 MRS files.
"""

import h5py
import numpy as np

# -------------------------------------------------------------------
def load(filename) -> (np.ndarray, dict):
    with h5py.File(filename, 'r') as hf:
        data = hf['data'][:]
    shape = data.shape
    assert shape[1] == 2 # represent real and imag values
    assert shape[2] == 1 # Should stay to 1 in our case
    data.shape = (shape[0], shape[1], shape[3])
    nbPts = shape[3]
    nbSignal = shape[0]
    meta = _makeHeader(nbPts, nbSignal)

    # transform data into a complex 2D array
    sig = np.empty((nbSignal, nbPts), complex)
    for i, val in enumerate(data):
        sig[i].real = val[0]
        sig[i].imag = val[1]

    del data
    return sig, meta

# -------------------------------------------------------------------
def _makeHeader(size, nbSig) -> dict:
    info = {}
    info['size'] = size
    info['step'] = None
    info['transFreq'] = None
    info['B0'] = None
    info['ppmref'] = None
    info['nbSignal'] = nbSig
    return info

# -------------------------------------------------------------------
def save(sig, filename) -> None:
    """
    NOT IMPLEMENTED
    """
    pass

# -------------------------------------------------------------------
if __name__ == "__main__":
    pass
