# -*-coding:Utf-8 -*

# ---------------------------------------------------------------------
#
# Copyright (c) CREATIS : www.creatis.insa-lyon.fr
# Centre de Recherche en Acquisition et Traitement de l'Image pour la Santé
# 
#  This software is governed by the CeCILL-B license under French law and
#  abiding by the rules of distribution of free software. You can  use,
#  modify and/ or redistribute the software under the terms of the CeCILL-B
#  license as circulated by CEA, CNRS and INRIA at the following URL
#  www.cecill.info/licences/Licence_CeCILL-B_V1-en.html
#  or in the file LICENCE.txt.
#
#  As a counterpart to the access to the source code and  rights to copy,
#  modify and redistribute granted by the license, users are provided only
#  with a limited warranty  and the software's author,  the holder of the
#  economic rights,  and the successive licensors  have only  limited
#  liability.
#
#  The fact that you are presently reading this means that you have had
#  knowledge of the CeCILL-B license and that you accept its terms.
# ------------------------------------------------------------------------

import tempfile

"""
Parser for QUEST parameters files.
"""

# =======================================================================

class QuestParam:
    """
    Allow to load and save parameters from/into a text file following the syntax
    param=value. Comments begin with a #.
    """

    # -------------------------------------------------------------------
    def __init__(self, path=None)->None:
        self.met = []
        self.macro = []
        self.rel = []
        self.fit = {}
        if path:
            self.load(path)

    # -------------------------------------------------------------------
    def load(self, path)->None:
        """
        Try to read the file at path. met, macro and rel parameters are fetch
        into attributes with the same name. All the others are into a
        dictionary following the syntax dict[name_param] = value
        """

        self.met = []
        self.macro = []
        self.rel = []
        self.fit = {}
        with open(path, 'r') as qfile:
            for line in qfile:
                try:
                    line = line.split('#')[0]
                    line = line.strip()
                    if line != "":
                        line = line.split('=')
                        line[0] = line[0].strip()
                        line[1] = line[1].strip()
                        if line[0] == "met":
                            self.met.append(line[1].split())
                        elif line[0] == "macro":
                            self.macro.append(line[1].split())
                        elif line[0] == "rel":
                            self.rel.append(line[1].split())
                        else:
                            self.fit[line[0]] = line[1].split()
                except IndexError:
                    raise SyntaxError("QUEST file does not respect param=value syntax")

    # -------------------------------------------------------------------
    def save(self, path="./paramFile.quest")->None:
        """
        By default, save parameters in the current directory as
        paramFile.quest
        """

        with open(path, 'w') as qfile:
            qfile.write("#! Basis Spectra\n")
            for line in self.met:
                qfile.write("met=")
                for word in line:
                    qfile.write(word + " ")
                qfile.write("\n")
            for line in self.macro:
                qfile.write("macro=")
                for word in line:
                    qfile.write(word + " ")
                qfile.write("\n")

            qfile.write("\n#! Metabolite fitting constraints\n")
            for line in self.rel:
                qfile.write("rel=")
                for word in line:
                    qfile.write(word + " ")
                qfile.write("\n")

            qfile.write("\n#! Fitting parameters\n")
            for key in self.fit:
                qfile.write(key + "=")
                for word in self.fit[key]:
                    qfile.write(word + " ")
                qfile.write("\n")

    # -------------------------------------------------------------------
    def save_temp(self)->str:
        """
        Save parameters in a temporary file. Return the path of this file
        """
        path = tempfile.mkstemp('.quest')
        self.save(path[1])
        return path[1]

# =======================================================================

if __name__ == "__main__":
    pass
