# -*-coding:Utf-8 -*

# ---------------------------------------------------------------------
#
# Copyright (c) CREATIS : www.creatis.insa-lyon.fr
# Centre de Recherche en Acquisition et Traitement de l'Image pour la Santé
# 
#  This software is governed by the CeCILL-B license under French law and
#  abiding by the rules of distribution of free software. You can  use,
#  modify and/ or redistribute the software under the terms of the CeCILL-B
#  license as circulated by CEA, CNRS and INRIA at the following URL
#  www.cecill.info/licences/Licence_CeCILL-B_V1-en.html
#  or in the file LICENCE.txt.
#
#  As a counterpart to the access to the source code and  rights to copy,
#  modify and redistribute granted by the license, users are provided only
#  with a limited warranty  and the software's author,  the holder of the
#  economic rights,  and the successive licensors  have only  limited
#  liability.
#
#  The fact that you are presently reading this means that you have had
#  knowledge of the CeCILL-B license and that you accept its terms.
# ------------------------------------------------------------------------

"""
Module with functions to read .txt files from 
QUEST output.
"""

def readQuest2Output(pathname) -> dict:
    """
    Read the _quest2.txt file from QUEST output
    Args : 
         - pathname : path to the f_quest2.txt file
    Return : 
         - res : a dictionary with metabolite's names as keys and a 2D array as value. The 
                 2D array store 7 info from the file for each voxel processed ~ res['met'][voxel][0-6]
                 
                 Info => 0:Amplitude ; 1:AmpSD ; 2:Phase ; 3:Damping ; 4:DFreq ; 5:Noise ; 6:EstCRB
    """

    res = {}
    met = ""
    vox = 0
    with open(pathname, 'r') as qfile:
        for line in qfile:
            line = line.strip()
            if line == "":
                continue

            line = line.split()
            if line[0] == "Metabolite:":
                met = line[1]
                res[met] = []
                vox = 0
                qfile.readline() # go to the line after because this one is only parameter's names
                continue
            
            res[met].append([])
            res[met][vox] = []
            for elm in line[2:]:
                res[met][vox].append(elm)
            vox += 1

    return res

if __name__ == '__main__':
    pass
