# ---------------------------------------------------------------------
#
# Copyright (c) CREATIS : www.creatis.insa-lyon.fr
# Centre de Recherche en Acquisition et Traitement de l'Image pour la Santé
# 
#  This software is governed by the CeCILL-B license under French law and
#  abiding by the rules of distribution of free software. You can  use,
#  modify and/ or redistribute the software under the terms of the CeCILL-B
#  license as circulated by CEA, CNRS and INRIA at the following URL
#  www.cecill.info/licences/Licence_CeCILL-B_V1-en.html
#  or in the file LICENCE.txt.
#
#  As a counterpart to the access to the source code and  rights to copy,
#  modify and redistribute granted by the license, users are provided only
#  with a limited warranty  and the software's author,  the holder of the
#  economic rights,  and the successive licensors  have only  limited
#  liability.
#
#  The fact that you are presently reading this means that you have had
#  knowledge of the CeCILL-B license and that you accept its terms.
# ------------------------------------------------------------------------

import tarfile
from threading import Thread
import subprocess
import time
import os
import os.path
import shutil
from glob import glob
from wx import CallAfter

from . import vip

"""
Module for thread executing Quest either locally or on VIP.
"""

# =======================================================================

class QuestThread(Thread):
    """
    TODO:
    gestion des erreurs dans le thread OU faire en sorte qu'il n'y ai jamais 
    d'erreurs dans l'execution du thread
    """

    # -------------------------------------------------------------------
    def __init__(self, qpath, callback, param_path, files, wts, cwd):
        super().__init__()
        self.callbk = callback
        self.pp = param_path
        self.files = files
        self.wts = wts
        self.cwd = cwd
        self.qpath = qpath
        print(self.qpath)
        self.start()

    # -------------------------------------------------------------------
    def run(self):
        param_path = self.pp
        where_to_save = self.wts
        if os.name == 'nt':
            command = self.qpath + "\cquestCml.bat -f " + param_path + " "
        else:
            command = self.qpath + "cquestCml     -f " + param_path + " "
#            command = ["./cquestCml ", "-f ", param_path, " ", ""]
#        print(command)

        try:
            for sig_path in self.files:
                #command[4] = sig_path
                command = command + sig_path
                prefix = os.path.basename(sig_path)
                prefix = os.path.splitext(prefix)[0]
                folder_name = os.path.join(where_to_save, prefix)
 
                print(command)
                print(self.qpath)
                proc = subprocess.Popen(command, cwd=self.qpath)
                proc.wait()
                # basicelly where QUEST actually saved its output files
                created_path = os.path.dirname(sig_path)

                # copy the original signal
                shutil.copyfile(sig_path,
                                os.path.join(folder_name, prefix + '_quest_orig.mrui'))
                # copy the parameters file for fit details display
                shutil.copyfile(param_path,
                                os.path.join(folder_name, prefix + '_quest_param.txt'))
                # copy the files created by QUEST
                for created_file in glob(os.path.join(created_path,
                                                           prefix+'_*')):
                    new_file = os.path.basename(created_file)
                    os.rename(created_file, os.path.join(folder_name, new_file))
        finally:
            os.remove(param_path)

        CallAfter(self.callbk)

# =======================================================================

class VipThread(Thread):
    """
    TODO:
    gestion des erreurs dans le thread OU faire en sorte qu'il n'y ai jamais 
    d'erreurs dans l'execution du thread
    """

    # -------------------------------------------------------------------
    def __init__(self, callback, apikey, param_path, base_path, files, wts):
        super().__init__()
        self.callback = callback
        self.apikey = apikey
        self.pp = param_path
        self.bp = base_path
        self.files = files
        self.wts = wts

        self.start()

    # -------------------------------------------------------------------
    def run(self):
        vip.setApiKey(self.apikey) 

        path_proc_vip = vip.create_dir_smart("/vip/Home/MRSVIEWER_cquest") + '/'
        path_basis_vip = path_proc_vip+'basis.zip'
        path_param_vip = path_proc_vip+'param.txt'

        #upload basis + param file
        vip.upload(self.bp, path_basis_vip)
        vip.upload(self.pp, path_param_vip)
        #upload a signal and launch cquest on it
        ids = []
        for sig_local in self.files:
            sig_path = path_proc_vip + os.path.basename(sig_local)
            vip.upload(sig_local, sig_path)
            name_exec = "MRSVIEWER_" + \
                       os.path.splitext(os.path.basename(sig_local))[0]
            id_ = self.init_cquest(name_exec, path_proc_vip,
                                  path_param_vip, sig_path,
                                  path_basis_vip)
            ids.append(id_)

        # Wait for pipelines
        self.__wait(ids)

        # Download results
        for id_ in ids:
            list_res = vip.get_exec_results(id_)
            for vip_path in list_res:
                start = vip_path.find("/rest/path/") + 10
                end = vip_path.find("?action=")
                vip_path = vip_path[start:end] #keep only '/vip/.../file.ext'
                prefix = vip_path.split('/')[-1].split('.')[0]
                dir_local = os.path.join(self.wts, prefix)
                vip.download(vip_path, dir_local + '.tgz')
                #extract
                #the dir local is supposed te exist already, cf quest/interface.py ->_processVip
                if not os.path.exists(dir_local):
                    os.mkdir(dir_local)

                tarfile.open(dir_local+'.tgz', 'r').extractall(dir_local)
                os.remove(dir_local+'.tgz')
                fname = os.path.join(dir_local, prefix)
                os.rename(fname + '.mrui', fname + '_quest_orig.mrui')


        # Clean local param.txt n basis
        os.remove(self.pp)
        os.remove(self.bp) #the basis zipped folder
        shutil.rmtree(self.bp[:-4]) #the basis folder

        # It's done
        CallAfter(self.callback)

    # -------------------------------------------------------------------
    def __wait(self, ids):
        testing = ids[:]
        while len(testing) != 0:
            time.sleep(20*len(testing))
            testing = [i for i in testing if vip.is_running(i)]

    # -------------------------------------------------------------------
    def init_cquest(self, name, result_dir, param_file, data_file,
                    zip_folder)-> str:
        """
        Args :
            - name : short description of this execution
            - result_dir: where to store result on VIP
            - param_file: where is stored parameters on VIP
            - data_file: path to the signal to preocess, on VIP
            - zip_folder: the basis with meta and macro, on VIP
        Return :
            The id of this execution (used for function like is_running)
        """
        values = {
            "results-directory": result_dir,
            "parameter_file": param_file,
            "data_file": data_file,
            "zipped_folder": zip_folder
        }
        return vip.init_exec("CQUEST/0.1", name, values)

# =======================================================================

if __name__ == '__main__':
    pass
