# -*-coding:Utf-8 -*

# ---------------------------------------------------------------------
#
# Copyright (c) CREATIS : www.creatis.insa-lyon.fr
# Centre de Recherche en Acquisition et Traitement de l'Image pour la Santé
# 
#  This software is governed by the CeCILL-B license under French law and
#  abiding by the rules of distribution of free software. You can  use,
#  modify and/ or redistribute the software under the terms of the CeCILL-B
#  license as circulated by CEA, CNRS and INRIA at the following URL
#  www.cecill.info/licences/Licence_CeCILL-B_V1-en.html
#  or in the file LICENCE.txt.
#
#  As a counterpart to the access to the source code and  rights to copy,
#  modify and redistribute granted by the license, users are provided only
#  with a limited warranty  and the software's author,  the holder of the
#  economic rights,  and the successive licensors  have only  limited
#  liability.
#
#  The fact that you are presently reading this means that you have had
#  knowledge of the CeCILL-B license and that you accept its terms.
# ------------------------------------------------------------------------

"""
Module that implement the Observer pattern.
Classes for the Observer pattern : Observer and Subject
"""

# =======================================================================

class Observer:
    # -------------------------------------------------------------------
    def update(self, *args, **kwargs):
        raise NotImplementedError("Interface : childrens should have" \
                                  "implemented this")

# """""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""

class Subject:
    # -------------------------------------------------------------------
    def __init__(self):
        self._observers = []

    # -------------------------------------------------------------------
    def registerObserver(self, observer):
        assert isinstance(observer, Observer)
        self._observers.append(observer)

    # -------------------------------------------------------------------
    def unregisterObserver(self, observer):
        for i, obs in enumerate(self._observers):
            if obs is observer:
                del self._observers[i]

    # -------------------------------------------------------------------
    def notify(self, *args, **kwargs):
        for obs in self._observers:
            obs.update(*args, **kwargs)

# =======================================================================

if __name__ == '__main__':
    pass
